/* -------------------------------------------------------------------	*/
/* Copyright (C) 2000 Robert R. Bruner.  All rights reserved.		*/
/* Contact the author at						*/
/*                    rrb@math.wayne.edu				*/
/*                    Dept of Mathematics				*/
/*                    Wayne State University				*/
/*                    Detroit, Michigan  48202				*/
/*                    USA						*/
/*									*/
/* Version 1.6								*/
/* -------------------------------------------------------------------- */

#include <stdio.h>
#include <stdlib.h>

#include <errors.h>
#include <storage.h>
#include <io.h>
#include <steen.h>
#include <structur.h>
#include <util.h>

char Status_fmt[] = "%10d";   /* format of status in files Im and Ker */
char Diff_hdr[] = "%10d %10d";   /* format of header in Diff files    */


int main(argc,argv) int argc; char *argv[];
{
int g,degree,i,nread,gen,op,
    lgen,lop,                /* leading gen and op in OIm files        */
    istatus,kstatus,         /* status of image and kernel files       */
    s,t,                     /* bidegree to do                         */
    verb2=0,verbose=0,           /* if third arg is -w or -v resp.         */
    seqno,                   /* seqno of op in c                       */
    degree_done,             /* of Diff.s                              */
    *dims,                   /* dims of subspaces spanned by each gen  */
    totdim=0,                /* sum of these                           */
    **found,                 /* foreach gen,op 1 if found, 0 if not    */
    Im_dim=0,Ker_dim=0,      /* for consistency check                  */
    num_gen;                 /* number of gens in filt s               */

/* ------------------------------------------------------------------  */
FILE *Diff,                  /* Diff.s-1 and then Diff.s               */
     *Ker,                   /* Ker_s.t                                */
     *OIm;                   /* OIm_s.t                                */
char filename[20];


/* -------------- check calling sequence -----------------------------  */
if ((argc != 3) && (argc != 4)) {
  printf("\nWrong number %d of args  Should be 2 or 3.\n",argc-1);
  for (i=0;i<argc;i++) printf("%3d: %s\n",i,argv[i]);
  exit(1);
 }
/* ------------------------------ get s,t ------------------------------ */

sscanf(argv[1],"%d",&s);
sscanf(argv[2],"%d",&t);
verbose = (argc == 4) && (argv[3][0] == '-') && 
	   ((argv[3][1] == 'v') || (argv[3][1] == 'w')) && 
	     (argv[3][2] == '\0');
if (verbose) verb2 = (argv[3][1] == 'w');

if (verbose) printf("\nChecking bidegree %d %d \n",s,t);

/* --------------------- process Diff.s   ---------------------------  */
/* Check this filtration for completeness, and get the number of gens  */
/*  (= number of next gen).                                            */

sprintf(filename,"Diff.%d",s);
if ((Diff = fopen(filename,"r")) == NULL) {
   printf("\nFile not found: %s\n",filename);
   exit(1);
}

if ((nread = fscanf(Diff,Diff_hdr,&num_gen,&degree_done)) != 2) {
   printf("\nBad %s: read only %d variables from header.",
                  filename,nread);
   exit(1);
}

if (degree_done < t) {
   printf("\nBad %s: degree done (%d) less than t (%d).",
                  filename,degree_done,t);
   exit(1);
}

if (verbose) printf("\nDiff.%d done through degree %d\n",s,degree_done);

/* read Diff.s to get degrees of generators so we'll know how large  */
/*  the vector space they generate in degree t is                    */

dims = (int *)malloc(num_gen*sizeof(int));
found = (int **)malloc(num_gen*sizeof(int *));
if (verb2) printf("\nX%10d           num_gen",num_gen);

for (g=0; g<num_gen; g++) 
 { fscanf(Diff,"%d",&degree);
   skip_Element(Diff);
   totdim += (dims[g] = dimA(t-degree));
   if (verb2) printf("\nX%10d %10d  max op",g,dims[g]);
   if (dims[g] > 0) 
     { found[g] = (int *)malloc(dims[g]*sizeof(int));
       for (i=0; i<dims[g]; i++) found[g][i] = 0;
     }
 }

fclose(Diff);

/* done with Diff.s.  Now look at Ker and Im to see if all these     */
/*  elements have really been processed.                             */


/* -------------  open Ker_s.t  ----------------------------------   */

sprintf(filename,"Ker_%d.%d",s,t);
if ((Ker = fopen(filename,"r")) == NULL) {
     printf("\nError opening %s ",filename);
     exit(1);
}
fscanf(Ker,Status_fmt,&kstatus);
if (verbose) printf("\n Kernel status %d\n",kstatus);
if ((kstatus != 1) && (kstatus != 2)) 
   { printf("\nNo\nKernel status %d\n",kstatus); exit(1); }

while (fscanf(Ker,"%d %d",&gen,&op) != EOF)
{ if ((gen < 0) || (gen >= num_gen))
    { printf("\nBad gen %d (num_gen = %d)in ker\n",gen,num_gen);
      exit(1);
    }
  if ((op < 0) || (op >= dims[gen]))
    { printf("\nBad op %d (dims[%d] = %d)in ker\n",op,gen,dims[gen]);
      exit(1);
    }
  if (found[gen][op])
    { printf("\nDuplicate pair %d %d in kernel\n",gen,op);
      exit(1);
    }
  if (verb2) printf("\nX%10d %10d Ker",gen,op);
  found[gen][op] = 1;
  Ker_dim++;
  skip_Element(Ker);
}

fclose(Ker);

/* Kernel done.  Check OIm_s.t -------------------------------------  */


sprintf(filename,"OIm_%d.%d",s,t);
if ((OIm = fopen(filename,"r")) == NULL) {
     printf("\nError opening %s ",filename);
     exit(1);
}

while (fscanf(OIm,"%d %d %d %d",&lgen,&lop,&gen,&op) != EOF)
{ if ((gen < 0) || (gen >= num_gen))
    { printf("\nBad gen %d (num_gen = %d) in Im\n",gen,num_gen);
      exit(1);
    }
  if ((op < 0) || (op >= dims[gen]))
    { printf("\nBad op %d (dims[%d] = %d) in Im\n",op,gen,dims[gen]);
      exit(1);
    }
  if (found[gen][op])
    { printf("\nDuplicate pair %d %d in im\n",gen,op);
      exit(1);
    }
  if (verb2) printf("\nX%10d %10d OIm",gen,op);
  found[gen][op] = 1;
  Im_dim++;
  skip_Element(OIm);
  skip_Element(OIm);
}

fclose(OIm);

if (verbose) printf("\n");

/* Now look to see if everything was hit.                               */

for (g=0; g<num_gen; g++)
    for (i=0; i<dims[g]; i++)
       if (found[g][i] == 0) 
	 { printf("\nNo\nGen %d Op %d missed\n",g,i);
	   exit(1);
         }
if (totdim != Im_dim + Ker_dim) 
   printf("\nNo - inconsistency with totdim = %d\n%d\n%d\n",
	  totdim,Im_dim,Ker_dim);
else
printf("\nYes\n%d\n%d\n",Im_dim,Ker_dim);
return 0;
}

