/* -------------------------------------------------------------------	*/
/* Copyright (C) 2008 Robert R. Bruner.  All rights reserved.		*/
/* Contact the author at						*/
/*                    rrb@math.wayne.edu				*/
/*                    Dept of Mathematics				*/
/*                    Wayne State University				*/
/*                    Detroit, Michigan  48202				*/
/*                    USA						*/
/*									*/
/* Version 1.8								*/
/* -------------------------------------------------------------------- */

#include <stdio.h>
/*#include <malloc.h> */
#include <stdlib.h>

/* --------------------------------------------------------------- */
int main(argc,argv) int argc; char *argv[];
{

int i, new, bottom, numgen, newgen, gen, g, op, count;
int *degrees, *newname;
FILE *outputfile;
FILE *inputfile;

if (argc != 4) 
{
   fprintf(stderr,"\nUsage: collapse <Def1> <bottom> <Def>");
   fprintf(stderr,"\n  will produce a module definition file named <Def>");
   fprintf(stderr,"\n  which defines the module obtained from <Def1> by");
   fprintf(stderr,"\n  taking the submodule containing all elements of degree");
   fprintf(stderr,"\n  greater than <bottom>, effectively collapsing to a point\n");
   fprintf(stderr,"\n  the classes up to and including theis dimension.\n");
   exit(1);
}
if (NULL == (inputfile=fopen(argv[1],"r")))
{ fprintf(stderr,"\nCannot open module definition: %s\n",argv[1]);
  exit(1);
}
if (NULL == (outputfile=fopen(argv[3],"w")))
{ fprintf(stderr,"\nCannot open output file: %s\n",argv[3]);
  exit(1);
}
sscanf(argv[2],"%d",&bottom);

fscanf(inputfile, "%d", &numgen);

degrees=(int *)malloc(numgen * sizeof(int));
newname=(int *)malloc(numgen * sizeof(int));

for (gen=0;gen<numgen;gen++)
   fscanf(inputfile, "%d",&degrees[gen]);

printf("\nTranslation table\nold gen   new gen\n");

for (gen=newgen=0; gen<numgen; gen++)
{
  if (degrees[gen] > bottom) newname[gen] = newgen++;
  else newname[gen] = -1;
  printf("%7d   %7d\n",gen,newname[gen]);
}

fprintf(outputfile,"%d\n",newgen);

for (new=gen=0; gen < numgen; gen++)
{
  if (!(new & 15)) fprintf(outputfile,"\n");
  if (newname[gen] >= 0)
  {
    new++;
    fprintf(outputfile,"%d ",degrees[gen]);
  }
}
fprintf(outputfile,"\n\n");

while (EOF != fscanf(inputfile, "%d %d %d", &gen,&op,&count))
{
  if ((gen < 0 ) || (gen >= numgen))
  { fprintf(stderr,
      "Operation on nonexistent generator %d\n\n",gen);
    exit(1);
  } 
  if (degrees[gen] > bottom)
  {
    fprintf(outputfile,"%d %d %d ",newname[gen],op,count);

    for(i=0; i<count; i++)      
    {
      fscanf(inputfile, "%d", &g);  
      if ((g < 0 ) || (g >= numgen))
      { fprintf(stderr,
          "Operation result nonexistent: %d\n\n",g);
        exit(1);
      }
      if (op + degrees[gen] != degrees[g])
      { fprintf(stderr,
           "Degree error: Sq(%d)(%d) contains %d, ",op,gen,g);
	fprintf(stderr,
           "but deg(%d) = %d while deg(%d) = %d\n",gen,degrees[gen],
		g,degrees[g]);
        exit(1);
      }
      fprintf(outputfile,"%d ",newname[g]);
    }
    fprintf(outputfile,"\n");
  } else
  for(i=0; i<count; i++)      
      fscanf(inputfile, "%d", &g);  
}
fclose(inputfile);
fclose(outputfile);
return 0;
}

