/* -------------------------------------------------------------------	*/
/* Copyright (C) 2000 Robert R. Bruner.  All rights reserved.		*/
/* Contact the author at						*/
/*                    rrb@math.wayne.edu				*/
/*                    Dept of Mathematics				*/
/*                    Wayne State University				*/
/*                    Detroit, Michigan  48202				*/
/*                    USA						*/
/*									*/
/* Version 1.6								*/
/* -------------------------------------------------------------------- */

#include <stdio.h>
#include <stdlib.h>

#include <errors.h>

int main(argc,argv) int argc; char *argv[];

{

int n,lo,hi,i,g;
FILE *Out;
char *filename="Def";

/* -------------- check calling sequence -----------------------------  */
if ((argc != 3) && (argc != 4))
{
  fprintf(stderr,
    "\nUsage: makeR <lo> <hi> [<file>]\n");
  fprintf(stderr,
    "  writes a Def file for the quotient by the 0-cell of stunted\n");
  fprintf(stderr,
    "  projective space with cells in dimensions <lo> to <hi> in \n");
  fprintf(stderr,
    "  <file> if specified, or in Def if no file is specified.\n\n");
  exit(1);
}

/* ------------------ get range --------------------------------------- */

sscanf(argv[1],"%d",&lo);
sscanf(argv[2],"%d",&hi);

/* ------------------ check validity --------------------------------- */

if ((lo >= 0) || (hi <=0))
{
  fprintf(stderr,
    "Use makeP to make stunted projective space for a range which\n");
  fprintf(stderr,
    "does not include a 0-cell\n\n");
  exit(1);
}

/* ------------------ open Def --------------------------------------- */

if (argc == 4) filename = argv[3];
if ((Out = fopen(filename,"w")) == NULL)
{
  fprintf(stderr,"\nCannot open %s to write\n\n",filename);
  exit(1);
}

/* ------------------ Now write header  ------------------------------ */

fprintf(Out,"%d\n",hi-lo);

for (i=lo; i <= hi; i++)
if (i != 0)
{
  if (!(i & 15)) fprintf(Out,"\n");
  fprintf(Out,"%d ",i);
}
fprintf(Out,"\n\n");

/* ------------------ write Sq^n(g)   ------------------------------ */

for (g=lo; g < hi; g++)
if (g != 0)
for (n=1; n <= hi-g; n++)
if ((n+g != 0) && (n == (n&g)))
  fprintf(Out,"%d %d 1 %d\n",g-lo-(g>0),n,n+g-lo-((n+g)>0));

/* --------------------------- Done   ------------------------------ */


fclose(Out);
return(0);

}

