/* -------------------------------------------------------------------	*/
/* Copyright (C) 2000 Robert R. Bruner.  All rights reserved.		*/
/* Contact the author at						*/
/*                    rrb@math.wayne.edu				*/
/*                    Dept of Mathematics				*/
/*                    Wayne State University				*/
/*                    Detroit, Michigan  48202				*/
/*                    USA						*/
/*									*/
/* Version 1.9.3 Corrected use of permutation 				*/
/* -------------------------------------------------------------------- */

#include <stdio.h>
/*#include <malloc.h> */
#include <stdlib.h>

#include <errors.h>


/* --------------------------------------------------------------- */

int main(argc,argv) int argc; char *argv[];

{

int i, j, k, gen, op, count;
int dim;
int *deg;
int *perm, *inv;            /* perm[i] is the original number of
                               new generatotr i, and inv[i] is the 
                               new number of the generator originally
                               numbered i                             */

FILE *In, *Out;

/* -------------- check calling sequence -----------------------------  */
if (argc != 3)
{
  fprintf(stderr,
    "\nUsage: sortDef <Def> <newDef>  \n");
  fprintf(stderr,
    "  Sorts the generators of <Def> by degree (as needed by addgen)\n");
  fprintf(stderr,
    "  and writes the result on the file <newDef>.\n");
  fprintf(stderr,
    "  It also prints on stdout the key expressing the generators of \n");
  fprintf(stderr,
    "  the new version in terms of the original one.\n\n");
  fprintf(stderr,
    "Warning:  it overwrites <newDef>.\n\n");
  exit(1);
}

/* ------------------ open files --------------------------------------- */

if ((In = fopen(argv[1],"r")) == NULL)
{
  fprintf(stderr,"\nCannot open %s to read\n",argv[1]);
  exit(1);
}

if ((Out = fopen(argv[2],"w")) == NULL)
{
  fprintf(stderr,"\nCannot open %s to write\n",argv[2]);
  exit(1);
}

/* ------------------ Now get headers  ------------------------------ */

fscanf(In,"%d",&dim);
fprintf(Out,"%d\n",dim);

deg = (int *)malloc(dim*sizeof(int));

for (i=0; i < dim; i++)
  fscanf(In,"%d",&deg[i]);

/* ------------------ Now sort them   ------------------------------ */
/* Yeah, I know, bubble sort is slow, but it is easy and I only have */
/* do this once on a small set                                       */

perm = (int *) malloc(dim*sizeof(int));
for (i=0; i<dim; i++) perm[i] = i;

for (i=dim-1; i > 0; i--)
for (j=0; j < i; j++)
if (deg[perm[j]] > deg[perm[j+1]] )
{ /* swap them */
  k = perm[j];
  perm[j] = perm[j+1];
  perm[j+1] = k;
}

/* ----- Sorted.  Now create reverse map and write it out   -------- */

inv = (int *) malloc(dim*sizeof(int));
for (i=0; i<dim; i++) inv[perm[i]] = i;

printf("New Old (generator numbers)\n");
for (i=0; i < dim; i++)
{
  printf("%3d %3d\n",i, perm[i]);
  if (!(i&15)) fprintf(Out,"\n");
  fprintf(Out,"%d ",deg[perm[i]]);
}
fprintf(Out,"\n\n");


/* ----------- Now read in the action and translate it   ----------- */


  while (EOF != fscanf(In, "%d %d %d", &gen,&op,&count))
  { fprintf(Out,"%d %d %d ",inv[gen],op,count);
    for(i=0;i<count;)
    { fscanf(In, "%d", &gen);  
      fprintf(Out,"%d ",inv[gen]);
      if (!(++i&15)) fprintf(Out,"\n");
    }
    fprintf(Out,"\n");
  }

/* --------------------------- Done   ------------------------------ */

fclose(In);
fclose(Out);
return(0);
   
}

