/* -------------------------------------------------------------------	*/
/* Copyright (C) 2015 Robert R. Bruner.  All rights reserved.		*/
/* Contact the author at						*/
/*                    rrb@math.wayne.edu				*/
/*                    Dept of Mathematics				*/
/*                    Wayne State University				*/
/*                    Detroit, Michigan  48202				*/
/*                    USA						*/
/*									*/
/* Version 1.9								*/
/* -------------------------------------------------------------------- */

#include <stdio.h>
#include <stdlib.h>

#include <errors.h>
#include <tensor.h>

int main(argc,argv) int argc; char *argv[];
{

int slo,shi,nlo,nhi;
int s,g,stem,j,N;
int stem0,j0,N0;
int s0, g0, g1, count, n;
int maxstem,minstem;
int p;
float off,off0;
int **Num;
float radius=0.05;
float gap=0.07;
float margin=2.0;
float pgwidth,pgheight;
Tensor_shape  *posn, *shape;
FILE  *Shape, *Out, *himults;
char message[100];


/* -------------- check calling sequence -----------------------------  */
if ((argc != 9))
{
  printf("\nUsage: chart slo shi nlo nhi shape himults out label\n\n");
  printf("    creates a TeX file using TikZ with lower left corner at\n");
  printf("    (s,n) = (slo,nlo) and upper right corner at\n");
  printf("    (s,n) = (shi,nhi) using the file shape to define\n");
  printf("    points and himults to define the lines.\n");
  printf("    The output is written into out.\n");
  printf("    Chart is labelled by the string label.\n\n");
  exit(1);
}

/* ------------------ get corners  --------------------------------- */

sscanf(argv[1],"%d",&slo);
sscanf(argv[2],"%d",&shi);
sscanf(argv[3],"%d",&nlo);
sscanf(argv[4],"%d",&nhi);

/* ------------------ position dots  ----------------------------------- */

shape = fread_shape(argv[5],-1);
posn = fread_shape(argv[5],-1);


maxstem = minstem = shape->deg[0][0];
for (s=0; s<=shape->s; s++)
 for (g=0; g<shape->numgen[s]; g++)
 { if (maxstem < shape->deg[s][g]-s) maxstem = shape->deg[s][g]-s;
   if (minstem > shape->deg[s][g]-s) minstem = shape->deg[s][g]-s;
 }

Num = (int **) malloc(((++maxstem)-minstem)*sizeof(int *));
Num -= minstem;
                
		/*      Now Num[minstem] through Num[maxstem-1] exist	*/
for (n=minstem; n < maxstem; n++)
{  Num[n] = (int *) malloc((1+shape->s)*sizeof(int));
   for (s=0; s<= shape->s; s++) Num[n][s] = 0;
}

for (s=0; s<=shape->s; s++)
 for (g=0; g<shape->numgen[s]; g++)
 { stem = shape->deg[s][g]-s;
   posn->deg[s][g] = Num[stem][s]++;
 }

if ((Out = fopen(argv[7],"w")) == NULL)
{  sprintf(message,"\nCannot open file: %s\n",argv[7]);
   terminate(message,BAD_WORKFILE);
}

/* ------------------ Write header  ----------------------------------- */

fprintf(Out,"\\documentclass{article}\n\n");
fprintf(Out,"\\usepackage{tikz}\n");
fprintf(Out,"\\pagenumbering{gobble}\n");


pgwidth = (float)(nhi-nlo);
pgheight = (float)(shi-slo);

fprintf(Out,"\\usepackage{pgflibraryarrows}\n\n");
fprintf(Out,
        "\\usepackage[paperwidth=%5.1fcm,paperheight=%5.1fcm,margin=%5.1fcm]{geometry}\n",
           2*margin+pgwidth+2.0,2*margin+pgheight+1.0,margin); 



fprintf(Out,"\\begin{document}\n\n");

fprintf(Out,"\\begin{figure}\n\n");

fprintf(Out,"\\begin{tikzpicture}[scale=1]\n\n");

fprintf(Out,"\\clip (-1.5,-0.5) rectangle (%6.2f,%6.2f);\n",
            pgwidth+0.5,pgheight+0.5);

fprintf(Out,"\\draw[color=lightgray] (0,0) grid [step=4]  (%d,%d);\n\n",
             nhi-nlo,shi-slo);


fprintf(Out,"\\foreach \\n in {%d,%d,...,%d}\n{\n",nlo,nlo+4,nhi);
fprintf(Out,"\\def\\nn{\\n-%d};\n",nlo);
fprintf(Out,"\\node [below] at (\\nn,0) {$\\n$};\n}\n\n");

fprintf(Out,"\\foreach \\s in {%d,%d,...,%d}\n{\n",slo,slo+4,shi);
fprintf(Out,"\\def\\ss{\\s-%d};\n",slo);
fprintf(Out,"\\node [left] at (-0.4,\\ss) {$\\s$};\n}\n\n");


/* Drawing is logically (nlo,slo) to (nhi,shi), but physically translated
   to (0,0) to (nhi-nlo,shi-slo)                                         */

/* ------------------ Draw dots  ----------------------------------- */

/* If there are multiple dots in a bidegree they are put on
a line of slope -1 arranged from right to left, centered at
the (n,s) coordinate.
This is reversed from the first version of chart.c, but seems
to give fewer crossing lines.  It is completely different from
the old display program, which zigzagged back and forth in the
attempt to keep things nicely centered, but which caused all
sorts of odd artifacts.                                              */

for (s=slo; s<=shi; s++)
for (g=0; g<shape->numgen[s]; g++)
 { stem = shape->deg[s][g]-s;
   if ((stem >= nlo) && (stem <= nhi))
   {
       N = Num[stem][s];
       j = posn->deg[s][g];
       off = -gap*((float)(2*j+1 - N));
       fprintf(Out,"\\draw [fill] (%5.2f,%5.2f) circle [radius=%4.2f];\n",
                   off + (float)(stem-nlo),
                   -off + (float)(s-slo),
		   radius);
       fprintf(Out,"\\node [left] at  (%5.2f,%5.2f) {\\tiny{%d}};\n\n",
                   off + (float)(stem-nlo),
                   -off + (float)(s-slo),
		   g);
   }
 }

/* ------------------ Now draw lines  ----------------------------------- */

fprintf(Out,"\n\n");


if ((himults = fopen(argv[6],"r")) == NULL)
{  sprintf(message,"\nCannot open file: %s\n",argv[6]);
   terminate(message,BAD_WORKFILE);
}

while (EOF != fscanf(himults,"%d %d %d %d %d",&s,&g,&s0,&g0,&p))
{
  stem = shape->deg[s][g]-s;
  stem0 = shape->deg[s0][g0]-s0;
  if (((s >= slo) && (s <= shi) && (stem >= nlo) && (stem <= nhi)) ||
      ((s0 >= slo) && (s0 <= shi) && (stem0 >= nlo) && (stem0 <= nhi)))
  {
    N = Num[stem][s];
    j = posn->deg[s][g];
    off = -gap*(float)(2*j+1-N);
    N0 = Num[stem0][s0];
    j0 = posn->deg[s0][g0];
    off0 = -gap*(float)(2*j0+1-N0);

    /* himults has the higher stem element first for hi mults,
       so we do differentials in reverse:  the target is s,g
       and the source is s0,g0.
    */

    if (s < s0)             /* differential  */
    {
       fprintf(Out,
            "\\draw [->] (%5.2f,%5.2f) --(%5.2f,%5.2f);\n",
                   off + (float)(stem-nlo),
                   -off + (float)(s-slo),
                   gap + off0 + (float)(stem0-nlo),
                   -gap -off0 + (float)(s0-slo));

		   /* Need offset to see arrowhead */

    }
    else                    /* multiplication  */
    {
    if (p == 2)
       fprintf(Out,
            "\\draw [dashed]  (%5.2f,%5.2f) --(%5.2f,%5.2f);\n",
                   off + (float)(stem-nlo),
                   -off + (float)(s-slo),
                   off0 + (float)(stem0-nlo),
                   -off0 + (float)(s0-slo));
    else if ((p == 0) || (p == 1))
       fprintf(Out,
            "\\draw (%5.2f,%5.2f) --(%5.2f,%5.2f);\n",
                   off + (float)(stem-nlo),
                   -off + (float)(s-slo),
                   off0 + (float)(stem0-nlo),
                   -off0 + (float)(s0-slo));
    }
  }
}

fprintf(Out,"\n\n");

/* ------------------ Now close files  ----------------------------------- */

fprintf(Out,"\\end{tikzpicture}\n");
fprintf(Out,
        "\\caption{%s,\\,\\, $%d \\leq n \\leq %d,\\,\\, %d \\leq s \\leq %d$}\n",
	argv[8],nlo,nhi,slo,shi);
fprintf(Out,"\\end{figure}\n\n");

fprintf(Out,"\\end{document}\n\n");

fclose(Out);
fclose(himults);
   
return 0;
}

