/* -------------------------------------------------------------------	*/
/* Copyright (C) 2000 Robert R. Bruner.  All rights reserved.		*/
/* Contact the author at						*/
/*                    rrb@math.wayne.edu				*/
/*                    Dept of Mathematics				*/
/*                    Wayne State University				*/
/*                    Detroit, Michigan  48202				*/
/*                    USA						*/
/*									*/
/* Version 1.65								*/
/* -------------------------------------------------------------------- */

/* This checks consistency of A(2) = <Sq^1, Sq^2, Sq^4> modules.	*/
/* It suffices to test Adem relations for 10 Adem relations, listed 	*/
/* in the vectors adema and ademb.					*/

#include <stdio.h>
/* #include <malloc.h> */
#include <stdlib.h>
#include <util.h>
#include <steen.h>
#include <structur.h>
#include <storage.h>
#include <io.h>
#include <errors.h>

typedef struct Steen_Op {
		    int first_sq;
	struct Steen_Op	*first_rest;
	struct Steen_Op	*other_terms; } Steen_Op;

typedef struct module_element {
			       int first;
	     struct module_element *next;} module_element;

module_element ***sq_value=NULL;
int maxdeg, numgen, *degrees;
int initialized=0;

int adema[] = {1, 1, 1, 1, 2, 2, 2, 3, 4, 5};
int ademb[] = {1, 2, 4, 6, 2, 3, 4, 5, 4, 3};

/* ------------------------------------------------------------------ */
module_element * copy_module_element(x) module_element *x;
{ module_element *z;

if (x == NULL) return(NULL);

z = (module_element *) malloc(sizeof(module_element));
if (z == NULL)
{
  fprintf(stderr,"\nOut of memory\n");
  exit(1);
}
z->first = x->first;
z->next = copy_module_element(x->next);
return(z);
}

/* ------------------------------------------------------------------ */
module_element * insert_gen(g,e)  int g; module_element *e;
{
  if ((e==NULL) || g < e->first)
  {
    module_element *e1=(module_element *)malloc(sizeof(module_element));

    e1->first=g;
    e1->next=e;
    return(e1);
  }
  if (g == e->first)  /*bad data file */
  {
    fprintf(stderr,"bad data file: Def\n");
    return (e->next);
  }
  e->next=insert_gen(g,e->next);
  return(e);
}

/* ------------------------------------------------------------------ */
void initialize(inputfile) FILE *inputfile;
{
int i, gen, op, count, g, lastdeg;


  fscanf(inputfile, "%d", &numgen);

  degrees=(int *)malloc(numgen * sizeof(int));
  for (gen=0;gen<numgen;gen++)
    fscanf(inputfile, "%d",&degrees[gen]);
  for (lastdeg=degrees[0], gen=1; gen < numgen; lastdeg=degrees[gen++])
    if (lastdeg > degrees[gen])
    {
      fprintf(stderr,
        "\nGenerators out of order: degree[%d]=%d and degree[%d]=%d\n",
        gen-1,lastdeg,gen,degrees[gen]);
      fprintf(stderr,"Run sortDef to reorder them.\n\n");
      exit(1);
    }
  maxdeg=degrees[numgen-1];

  sq_value=(module_element ***)malloc(numgen*sizeof(module_element **));
  for (gen=0;gen<numgen;gen++)
  {
    sq_value[gen]=(module_element **)malloc(
	 (1+maxdeg - degrees[gen])*sizeof(module_element *));
    sq_value[gen][0] = (module_element *)malloc(sizeof(module_element));
    sq_value[gen][0]->first = gen;
    sq_value[gen][0]->next = NULL;
    for(i=1;i<=maxdeg - degrees[gen];i++)
      sq_value[gen][i]=NULL;
  }
    		/* sq_value[gen][op]'s are initially NULL 	*/
		/* except Sq(0) is the identity op  		*/

  while (EOF != fscanf(inputfile, "%d %d %d", &gen,&op,&count))
  if ((gen < 0 ) || (gen >= numgen))
  { fprintf(stderr,
      "Operation on nonexistent generator %d\n\n",gen);
    exit(1);
  } else
  if (count < 1) 
  { fprintf(stderr,
      "Bad count: %d\n\n",count);
     exit(1);
  } else
  if (op < 1) 
  { fprintf(stderr,
      "Bad Steenrod operation: %d\n\n",op);
     exit(1);
  } else
    for(i=0;i<count;i++)      
    { fscanf(inputfile, "%d", &g);  
      if ((g < 0 ) || (g >= numgen))
      { fprintf(stderr,
          "Operation result nonexistent: %d\n\n",g);
        exit(1);
      } 
      if (op < 1) 
      { fprintf(stderr,
          "Bad Steenrod operation: %d\n\n",op);
        exit(1);
      }
      if (op + degrees[gen] != degrees[g])
      { fprintf(stderr,
          "Degree error: Sq(%d)(%d) contains %d, ",op,gen,g);
	fprintf(stderr,
           "but deg(%d) = %d while deg(%d) = %d\n",gen,degrees[gen],
		g,degrees[g]);
      } else
      sq_value[gen][op]=insert_gen(g,sq_value[gen][op]);
    }
fclose(inputfile);
initialized=1;
return;
}

/* ------------------------------------------------------------------ */
int mod_basis(t,adegree) int t, **adegree;
{ int i,count;


for (count = i = 0; i < numgen; i++)
  if (degrees[i] <= t) count++;

*adegree = degrees;
return(count);
}

/* ------------------------------------------------------------------ */
void print_module_element(e)  module_element *e;
{
  char c;
  module_element *i=NULL;

  if (e != NULL)
    for(i=e, c=' ';i != NULL;i=i->next,c='+')
      printf("%c%d",c,i->first);
  printf(" ");
  return;
}

/* --------------------------------------------------------------- */
module_element * modadd(x,y) module_element *x,*y;
{
  module_element *x1=NULL,*y1=NULL;
				   /* destroys x and y as it adds */
  if (x==NULL) return(y);
  if (y==NULL) return(x);
  if (x->first < y->first)
  {
    x->next = modadd(x->next,y);
    return(x);
  }
  if (x->first > y->first)
  {
    y->next = modadd(x,y->next);
    return(y);
  }
  /* first terms are equal, hence cancel */
  x1=x->next;
  y1=y->next;
  free(x);
  free(y);
  return(modadd(x1,y1));
}

/* --------------------------------------------------------------- */
module_element * actone(op,x) int op; module_element *x;
{
  module_element *e=NULL,*f=NULL;
  char errmess[100];

  if (x==NULL)
    return(x);
  if ((x->first >= numgen) || (x->first < 0))
  { fprintf(stderr,
            "\nBad gen %d out of range %d to %d in actone\n",
	    x->first,0,numgen-1);
    exit(MODERR);
  }
  if (op + degrees[x->first] > maxdeg)
    e=NULL;
  else
    e = copy_module_element(sq_value[x->first][op]);
  f=actone(op,x->next);  /* if op is acting on an element x which is the
			    sum of more than one element, say
			    x1 + x2 + ... + xn, then f will equal op
			    acting on x2 + ... + xn                       */
  return(modadd(e,f));
}

/* --------------------------------------------------------------- */
int main(argc,argv) int argc; char *argv[];
{

int i,a,b,c,gen;
module_element *e,*f;
char *filename="Def";
FILE *inputfile;

if (argc == 2) filename = argv[1];
if (NULL == (inputfile=fopen(filename,"r")))
{ fprintf(stderr,"\nCannot open module definition: %s\n",filename);
  exit(BAD_FILE);
}

initialize(inputfile);


for (gen=0; gen < numgen; gen++)
for (i=0; i < 10; i++)
{
a = adema[i];
b = ademb[i];

e = (module_element *) malloc(sizeof(module_element));
e->first = gen;
e->next = NULL;

f = actone(a,actone(b,e));
for (c= (a<b)? 0:a-b+1; c <= a/2; c++)
 if (((a-2*c) & (b+c-1-a)) == 0)
  f = modadd(f,actone(a+b-c,actone(c,e)));

if (f != NULL)
{  printf("Adem rel (Sq(%d)Sq(%d) + ... )(gen %d) = ",a,b,gen);
   print_module_element(f);
   printf(", not zero.\n");
}
}
return 0;
}
