/* -------------------------------------------------------------------	*/
/* Copyright (C) 2000 Robert R. Bruner.  All rights reserved.		*/
/* Contact the author at						*/
/*                    rrb@math.wayne.edu				*/
/*                    Dept of Mathematics				*/
/*                    Wayne State University				*/
/*                    Detroit, Michigan  48202				*/
/*                    USA						*/
/*									*/
/* Version 1.6								*/
/* -------------------------------------------------------------------- */

#include <stdio.h>
#include <stdlib.h>

#include <errors.h>
#include <storage.h>
#include <io.h>
#include <steen.h>
#include <module.h>
#include <structur.h>
#include <util.h>

char Status_fmt[] = "%10d";   /* format of status in files Im and Ker */
char Diff_hdr[] = "%10d %10d";   /* format of header in Diff files    */


int main(argc,argv) int argc; char *argv[];
{
int i,nread,numops,deg,dim,found,
    s,t,                     /* bidegree to do                         */
    seqno,                   /* seqno of op in dg                      */
    degree_done,             /* of Diff.s-1 then of Diff.s             */
    dim_codomain,            /* dim over A of codomain                 */
    image_dim,               /* dim over A of codomain in degree < t   */
    dim_domain,              /* dim over A of domain                   */
    *degree;                 /* internal degree of gens of C.s-1       */
/* ------------------------------------------------------------------- */
                             /* A generator of C.s is given by its     */
int g,g_degree;              /* (1) sequence number g, (2) degree      */
TERM **dg;                  /* g_degree, and (3) differential dg in   */
                             /* expanded form for ease of calculating  */
                             /*  dx = op*dg for each op.               */

                             /* An operation op to act on g is given   */
int opseqno;                 /* by it's (1) sequence number opseqno,   */
Internal_Op Op;              /* and (2) internal form Op.              */
/* ------------------------------------------------------------------- */

Element ***image_x, ***image_dx;
                             /* These are each arrays of arrays of     */
                             /* pointers to Elements, such that the    */
                             /* non-NULL image_dx[g][o] are a basis    */
                             /* for the image, and                     */
                             /*  d(image_x[g][o]) = image_dx[g][o]     */
                             /* Further, image_dx[g][o] = o * g +      */
                             /*  higher terms                          */

/* ------------------------------------------------------------------  */

Internal_Op Op2;             /* op in dg                               */
TERM **x,**dx;               /* x = op*g, dx = op * dg                 */
FILE *Diff,                  /* Diff.s-1 and then Diff.s               */
     *Ker,                   /* Ker_s.t                                */
     *Im;                    /* Im_s.t                                 */
char filename[20],message[100];

/* -------------- check calling sequence -----------------------------  */
if (argc != 3) {
  printf("\nWrong number %d of args  Should be 2.\n",argc-1);
  for (i=0;i<argc;i++) fprintf(stderr,"%3d: %s\n",i,argv[i]);
  exit(1);
 }
/* ------------------------------ get s,t ------------------------------ */

sscanf(argv[1],"%d",&s);
sscanf(argv[2],"%d",&t);

/* Check previous filtration (or module) for completness, get dimension  */
/*  and degrees of gens to compute A dim of the codomain                 */

if (s > 0) {
   sprintf(filename,"Diff.%d",s-1);
   if ((Diff = fopen(filename,"r")) == NULL) {
      sprintf(message,"\nFile not found: %s\n",filename);
      terminate(message,BAD_DIFF);
     }

   if ((nread = fscanf(Diff,Diff_hdr,&dim_codomain,&degree_done)) != 2) {
      sprintf(message,"\nBad %s: read only %d variables from header.",
                     filename,nread);
      terminate(message,BAD_DIFF);
     }

   if (degree_done < t) {
      sprintf(message,"\n%s incomplete.  Done through degree %d",
                     filename,degree_done);
      terminate(message,BAD_DIFF);
     }
/* read Diff.s-1 to get degrees of generators so we'll know how much   */
/*  space to allocate                                                  */

   degree = (int *)malloc(dim_codomain*sizeof(int));
   image_dim = 0;
   for (g=0; g<dim_codomain; g++) {
      fscanf(Diff,"%d",&degree[g]);
      skip_Element(Diff);
      if (degree[g] <= t) image_dim = 1 + g;
   }

   fclose(Diff);
                  /* done:  have dim_codomain, image_dim, and degrees */
}
                  /* s = 0   -    get these from the module instead   */
                  /*     (except dim_codomain, which is not needed)   */
else
   image_dim = mod_basis(t,&degree);

dx = (TERM **) malloc(image_dim * sizeof(TERM *));
dg = (TERM **) malloc(image_dim * sizeof(TERM *));
image_x = (Element ***) malloc(image_dim * sizeof(Element **));
image_dx = (Element ***) malloc(image_dim * sizeof(Element **));

/* allocate space for dx's terms now, since we know their degrees.     */
/* allocate space for dg's terms when g is read in (set all entries    */
/*    of dg to NULL now so that read_exp_Element can assume this)      */
/* allocate entries in image and set them to NULL                      */

for (g=0; g<image_dim; g++)
{  dg[g] = NULL;
   deg = t - degree[g];
   if ((numops = dimA(deg)))
   { dx[g] = get_TERM(numops);
     dx[g]->gen = g;
     dx[g]->deg = deg;
     image_x[g] = (Element **)malloc(numops * sizeof(Element *));
     image_dx[g] = (Element **)malloc(numops * sizeof(Element *));
     for (i=0; i<numops; i++) image_x[g][i] = image_dx[g][i] = NULL;
   }
   else    /* numops == 0  */
   { image_x[g] = NULL;
     image_dx[g] = NULL;
     dx[g] = NULL;
   }
}


/* Check this filtration for completeness, and get the number of gens  */
/*  (= number of next gen).  Open for reading only since Addgen will   */
/*  update the differential file for this filtration.                  */

sprintf(filename,"Diff.%d",s);
if ((Diff = fopen(filename,"r")) == NULL) {
   sprintf(message,"\nFile not found: %s\n",filename);
   terminate(message,BAD_DIFF);
}

if ((nread = fscanf(Diff,Diff_hdr,&dim_domain,&degree_done)) != 2) {
   sprintf(message,"\nBad %s: read only %d variables from header.",
                  filename,nread);
   terminate(message,BAD_DIFF);
}

if (degree_done < t - 1) {
   sprintf(message,"\n%s incomplete.  Done through degree %d",
                  filename,degree_done);
   terminate(message,BAD_DIFF);
}
if (degree_done > t - 1) {
   fprintf(stderr,"\n%s already complete through degree %d",
                  filename,degree_done);
   fprintf(stderr,"\nContinuing anyway.");
}

/* ----- Differential files are ok.  Create image and open kernel.   */
/*       Write status at start of Ker and Im ----------------------- */

sprintf(filename,"Ker_%d.%d",s,t);
if ((Ker = fopen(filename,"w")) == NULL) {
     sprintf(message,"\nError opening %s ",filename);
     terminate(message,BAD_KER);
}
fprintf(Ker,Status_fmt,0);

sprintf(filename,"Im_%d.%d",s,t);
if ((Im = fopen(filename,"w")) == NULL) {
     sprintf(message,"\nError opening %s ",filename);
     terminate(message,BAD_IM);
}
fprintf(Im,Status_fmt,0);

/* -----------------------  Main loop -----------------------------    */

/* Allocate space for x now and reuse, since it never changes size.    */
/* Allocate space for x's TERMs only as they appear, since we will     */
/* not know their degree (and hence size) until then.   Similarly,     */
/* allocate space for dg's TERMs when g is read since their degrees    */
/* and hence their sizes vary with g                                   */

x = (TERM **) malloc(dim_domain * sizeof(TERM *));
for (g=0; g < dim_domain; g++) x[g] = NULL;

for (g=0; g < dim_domain; g++, return_TERMs(dg,image_dim))
{

  if (fscanf(Diff,"%d",&g_degree) == EOF)
  { sprintf(message,
       "\nDifferential file incomplete. Reading gen number %d\n",g);
    terminate(message,BAD_DIFF);
  }

  if ((dim = dimA(t - g_degree)) <= 0)     /* No ops on g end in deg t */
  { skip_Element(Diff);                    /* Skip this dg and go to   */
    continue;                              /*   next gen.              */
  }

  if (read_exp_Element(Diff,dg,image_dim) == EOF)
  { sprintf(message,
       "\nDifferential file incomplete. Reading gen number %d\n",g);
    terminate(message,BAD_DIFF);
  }

  x[g] = get_TERM(dim);
  x[g]->gen = g;
  x[g]->deg = t - g_degree;

  for (opseqno = first_op(Op,t-g_degree);
         opseqno >= 0;
             opseqno = next_op(Op, opseqno))
  {
   /*     x = op * g                                                   */

   for (i=0; i<=g; i++) if (x[i] != NULL) zero_op(x[i]);
   x[g]->op[opseqno >> 3] = (0x80) >> (opseqno & 7);

   /*     dx = op * dg                                                 */
   /*                 If s==0, dg and dx lie in the module, which may  */
   /*                 not be free, so modact must get all of dg and    */
   /*                 dx at once so that gens may be mixed by the      */
   /*                 action of Op.  If s>0, then we're in the         */
   /*                 resolution, modules are free, and the action is  */
   /*                 simply multiplication in the algebra.            */

   if (s == 0) modact(image_dim,dx,t,Op,opseqno,dg,g_degree);
   else
   for (i=0; i<image_dim; i++) if (dx[i] != NULL)
   { zero_op(dx[i]);
     for (seqno = first_op_in(dg[i],Op2);
            seqno >= 0;
               seqno = next_op_in(dg[i],Op2,seqno))
         mult(Op,Op2,dx[i]);
   }

/*  Now reduce against the image so far.                               */

for (found=i=0; (!found) & (i < image_dim); i++)
for (seqno = first_seqno_in(dx[i]);
      (!found) & (seqno >= 0);
        seqno = next_seqno_in(dx[i],seqno) )
if (image_dx[i][seqno] != NULL)            /* cancel off leading term  */
  { Element *e; int j;

    for (e = image_x[i][seqno], j=0; j < e->len; j++)
      if (subtract(x[e->item[j]->gen],e->item[j]))
      {  fclose(Im); fclose(Ker); fclose(Diff);
         fprintf(stderr,"\nError subtracting item %d of image from x.  x is:\n",j);
         write_exp_Element(stderr,x,'s',g+1);
         fprintf(stderr,"\n and image[%d][%d] is:\n",i,seqno);
         write_Element(stderr,e,'s');
         terminate(" ",BAD_REDUCE);
       }
    for (e=image_dx[i][seqno], j=0; j < e->len; j++)
      if (subtract(dx[e->item[j]->gen],e->item[j]))
      {  fclose(Im); fclose(Ker); fclose(Diff);
         fprintf(stderr,"\nError subtracting item %d of image from dx.  dx is:\n",j);
         write_exp_Element(stderr,dx,'s',image_dim);
         fprintf(stderr,"\n and image[%d][%d] is:\n",i,seqno);
         write_Element(stderr,e,'s');
         terminate(" ",BAD_REDUCE);
       }
  }
else                                /* dx has a new leading term       */
  { image_dx[i][seqno] = condense(dx,image_dim);
    image_x[i][seqno] = condense(x,g+1);
    fprintf(Im,"\n%d %d ",g,opseqno);
    write_Element(Im,image_dx[i][seqno],'c');
    write_Element(Im,image_x[i][seqno],'c');
    found = 1;
  }
/* If we fall out the end of the loop, without setting found = 1, dx   */
/* will have been reduced to 0. Check to be sure that is true.         */

if (!found)
{
 for (i=0; i < image_dim; i++) if (is_not_zero(dx[i]))
  { fprintf(stderr,
            "\ndx is still nonzero after falling thru reduce loop.");
    fprintf(stderr,
            "\ngen,op is %d,%d.  dx is\n",g,opseqno);
    write_exp_Element(stderr,dx,'s',image_dim);
    terminate(" ",BAD_REDUCE);
  }

/* Ok: dx really is zero.                                              */

fprintf(Ker,"\n%d %d ",g,opseqno);
write_exp_Element(Ker,x,'c',g+1);
} /* end if !found       */

} /* end   for opseqno   */
} /* end   for g         */

/*  Main loop finished.  Write status codes at start of Im and Ker.   */

fclose(Diff);

fprintf(Im,"\n");
rewind(Im);
fprintf(Im,Status_fmt,1);
fclose(Im);
fprintf(Ker,"\n");
rewind(Ker);
fprintf(Ker,Status_fmt,1);
fclose(Ker);
return 0;

}


