/* -------------------------------------------------------------------	*/
/* Copyright (C) 2016 Robert R. Bruner.  All rights reserved.		*/
/* Contact the author at						*/
/*                    rrb@math.wayne.edu				*/
/*                    Dept of Mathematics				*/
/*                    Wayne State University				*/
/*                    Detroit, Michigan  48202				*/
/*                    USA						*/
/*									*/
/* Version 1.9								*/
/* -------------------------------------------------------------------- */

#include <stdio.h>
#include <stdlib.h>

#include <tensor.h>

int main(argc,argv) int argc; char *argv[];
{

Tensor_shape   *found;
FILE  *Shape;

int s,g,any;

/* -------------- check calling sequence -----------------------------  */
if ((argc != 2))
{
  printf("\nUsage: missing_gens Shape\n\n");
  printf("    reads the Shape file to determine the generators\n");
  printf("    of the resolution, then reads stdin, which should\n");
  printf("    be a list of pairs s g derived from the first two\n");
  printf("    entries in a map summary file ('all'),\n");
  printf("    to see which ones have been hit thus far.\n");
  printf("    It reports any that are missing.\n\n");
  printf("    Sample usage: \n");
  printf("         awk '{print $1,$2}' < all | ./missing_gens Shape |more\n\n");
  
  exit(1);
}


/* ------------------ get shape  ----------------------------------- */

found = fread_shape(argv[1],-1);

/* Set all gens to 0 : not found              */
for (s=0; s<=found->s; s++)
 for (g=0; g<found->numgen[s]; g++)
   found->deg[s][g] = 0;
 
/* ----- read from stdin the list of already hit generators ------ */

while (EOF != scanf("%d %d",&s,&g))
  found->deg[s][g] = 1;

/* ---------------------  Are any are missing ------------------  */

any = 0;
for (s=0; s<=found->s; s++)
 for (g=0; g<found->numgen[s]; g++)
   if (found->deg[s][g] ==  0)
      any = 1;
 
/* ----------------- report them if there are ------------------ */

if (!any)
    printf("\nNo missing generators\n\n");
else  
{   printf("\nMissing generators:\n");
    for (s=0; s<=found->s; s++)
     for (g=0; g<found->numgen[s]; g++)
       if (found->deg[s][g] ==  0)
          printf("%3d %3d\n",s,g);
}
}

