/* -------------------------------------------------------------------	*/
/* Copyright (C) 2008 Robert R. Bruner.  All rights reserved.		*/
/* Contact the author at						*/
/*                    rrb@math.wayne.edu				*/
/*                    Dept of Mathematics				*/
/*                    Wayne State University				*/
/*                    Detroit, Michigan  48202				*/
/*                    USA						*/
/*									*/
/* Version 1.6								*/
/* Version 1.8     moved fclose(Map) at line 281 into s1 > s loop       */
/*                 (line 279) to get rid of excess fclose error         */
/* -------------------------------------------------------------------- */



#include <stdio.h>
#include <stdlib.h>

#include <errors.h>
#include <storage.h>
#include <io.h>
#include <steen.h>
#include <structur.h>
#include <tensor.h>
#include <util.h>

char Diff_hdr[] = "%10d %10d";   /* format of header in Diff files    */


int main(argc,argv) int argc; char *argv[];
{
int s,g,s1,t,t1,	/* compute map(dg) for g in Domain_s1.t1	*/
			/*   where map has bidegree s,t			*/
    *done,		/* done[i] = 1 iff map(i) already computed (s1)	*/
    *mapped,		/* mapped[i] = 1 iff map(i) is read in (s1-1)	*/
    dimDomain,		/* dim over A of Domain_s1			*/
    status=0,		/* OR of     	0x100 = wrote Map,		*/
			/*		0x200= wrote ToLift,		*/
			/* 		0x400 = unmappable elts, so	*/
			/* 0 = up to date already			*/
			/* 1 - xFF = error code				*/
			/* 0x100 = up to date now			*/
			/* 0x200 = will be up to date after liftmap	*/
			/* 0x300 = ""					*/
			/* 0x400,0x500 = need to do filt s-1		*/
			/* 0x600,0x700 =  "" and liftmap		*/
    do_this_one,	/* 1 means all gens of dg mapped, so lift	*/
    dummy,nread,len,
    nonzero,		/* flag						*/
    imlen,		/* num gens in Domain_s mapped nontrivially	*/
    s0,g0,		/* used while reading Map file			*/
    seq1,seq2,		/* seq no.s of ops in dg and map(dg)		*/
    i,j,deg;		/* indices and degree of TERM as allocated	*/


Element ** im,		/* im[i] is map(gen i in Domain_s1-1)		*/
        *  dg,		/* d(g)					*/
	*  img,		/* map(one term of dg)				*/
       	*  imdg;	/* map(dg)					*/

Internal_Op Op1,Op2;

Tensor_shape *Dom_shape;	/* Domain and Codomain generators	*/
Tensor_shape *Cod_shape;	/*   and degress			*/

FILE *Diff,		/* domain Diff.s1				*/
     *Def,		/* map definition file				*/
     *Work,		/* work files organized by bidegree of image	*/
			/*   needed to lift them (ToLift_s.t)		*/
     *Map;		/* Map file					*/

char filename[100],
     message[200],
     Domain[100],
     Codomain[100],
     MapDir[100];

/* -------------- check calling sequence -----------------------------  */
if (argc != 2)
{
  printf("\nUsage: startmap s1 \n  Computes map(d(Domain_s1))\n");
  exit(-1);
}

/* ------------------------------------------------------------------   */
/*  get arg s1 and  read map definition file				*/

sscanf(argv[1],"%d",&s1);

if ((Def = fopen("Def","r")) == NULL)
{  
   terminate("\nMap definition file Def not found. \n", BAD_MAP);
}

fscanf(Def,"%d %d %s %s %s %d",&s,&t,Domain,Codomain,MapDir,&imlen);

if (s1 < s) exit(0);				/* Nothing to do	*/

/* ------------------------------------------------------------------   */
/* get shapes								*/

Dom_shape = fread_shape("../Shape",s1);
sprintf(filename,"../../%s/Shape",Codomain);
Cod_shape = fread_shape(filename,s1);

done = (int *) malloc(Dom_shape->numgen[s1] * sizeof(int));
for (i=0; i < Dom_shape->numgen[s1]; i++) done[i] = 0;
mapped = (int *) malloc(Dom_shape->numgen[s1-1] * sizeof(int));
for (i=0; i < Dom_shape->numgen[s1-1]; i++) mapped[i] = 0;
im = (Element **) malloc(Dom_shape->numgen[s1-1] * sizeof(Element *));

/* ------------------------------------------------------------------   */
/* get image of gens in filt s1 - 1 and note which gens in filt s1 	*/
/*   are already mapped							*/

if ((Map = fopen("Map","r")) == NULL)
{ terminate( "\nMap file not found.",BAD_MAP);
}

while (EOF != fscanf(Map,"%d %d",&s0,&g0))
{  
   if (s0 == s1)
   { done[g0] = 1; skip_Element(Map); }
   else if ((s0 == s1-1) && (s1 > s))
   {  mapped[g0] = 1;
      im[g0] = read_Element(Map);
   }
   else skip_Element(Map);
}

fclose(Map);


/* ------------------------------------------------------------------   */
/*  If s1 == s we just copy the definition from Def file and send	*/
/*    everything else to zero						*/

if (s1 == s)
{
  for (i=0; i < imlen; i++)
  {  fscanf(Def,"%d",&g);
     if (done[g])  skip_Element(Def);
     else { dg = read_Element(Def);
	    sprintf(filename,"../../%s/ToLift_0.%d",
		     Codomain,Dom_shape->deg[s1][g]-t);
            if ((Work = fopen(filename,"a")) == NULL)
            { sprintf(message,"\nFile not found: %s\n",filename);
              terminate(message,BAD_WORKFILE);
            }
            fprintf(Work,"\n../%s/%s/Map %d %d \n",Domain,MapDir,s,g);
            write_Element(Work,dg,'c');
            fclose(Work);
	    done[g] = 1;
	    status |= 0x200;	/* wrote ToLift	*/
          }
}
fclose(Def);

/* Now write out Map(g) = 0 for any generators g not already mapped
   or defined by Def	*/

if ((Map = fopen("Map","a")) == NULL)
{ terminate( "\nCannot open Map file to append.\n",BAD_MAP);
}

for (g=0; g < Dom_shape->numgen[s]; g++)
if (!(done[g]))
{
  status |= 0x100;
  fprintf(Map,"\n%d %d 0\n",s,g);
}

fclose(Map);
}

else		/*   s1 > s	*/
{   

/* ------------------------------------------------------------------   */
/* Open Diff.s1 and get positioned to read the diffs we need		*/

sprintf(filename,"../Diff.%d",s1);
if ((Diff = fopen(filename,"r")) == NULL)
{  sprintf(message,"\nFile not found: %s\n",filename);
   terminate(message,BAD_DIFF);
}

if ((nread = fscanf(Diff,Diff_hdr,&dimDomain,&dummy)) != 2)
{  sprintf(message,"\nBad %s: cannot read header.", filename);
   terminate(message,BAD_DIFF);
}

if (dimDomain > Dom_shape->numgen[s1])
{ dimDomain = Dom_shape->numgen[s1];
  fprintf(stderr,"\nShape file out of date.\n");
}
else if (dimDomain < Dom_shape->numgen[s1])
     { sprintf(message,"\nShape ahead of Diff.%d\n",s1);
       terminate(message,BAD_SHAPE);
     }

/* ------------------------------------------------------------------   */
/* Go ahead and open Map in case we have any 0's we don't have to lift	*/

if ((Map = fopen("Map","a")) == NULL)
       terminate("\nCannot open Map file to append.\n",BAD_MAP);

/* ------------------------------------------------------------------   */
/* Now get to it!							*/

for (g = 0; g < dimDomain; g++)
{
   fscanf(Diff,"%d",&t1);
   if (done[g])
     skip_Element(Diff);
   else
   {  dg = read_Element(Diff);
      do_this_one = 1;
      for (i=0; i < dg->len; i++)
	if (!mapped[dg->item[i]->gen])
	{  fprintf(stderr,"\nDiff of gen %d has unmapped gen %d.\n",
		    g,dg->item[i]->gen);
	   status |= 0x400;
	   do_this_one = 0;
	}
     if (do_this_one) {

     imdg = get_Element(Cod_shape->numgen[s1-s-1]);
     for (i=0; i < imdg->len; i++)
     if ((len = (int)dimA(deg = t1 - t - Cod_shape->deg[s1-s-1][i])) > 0)
     {  imdg->item[i] = get_TERM(len);
	imdg->item[i]->gen = i;
	imdg->item[i]->deg = deg;
	zero_op(imdg->item[i]);
     }
     else imdg->item[i] = NULL;

     for (i=0; i < dg->len; i++)
      if ((img = im[dg->item[i]->gen]) != NULL)
       for (j=0; j < img->len; j++)
        for (seq1 = first_op_in(dg->item[i],Op1);
              seq1 >= 0;
	        seq1 = next_op_in(dg->item[i],Op1,seq1))
	for (seq2 = first_op_in(img->item[j],Op2);
	      seq2 >= 0;
		seq2 = next_op_in(img->item[j],Op2,seq2))
	  mult(Op1,Op2,imdg->item[img->item[j]->gen]);

     /* If imdg is 0, we can add g to the Map file right now.		*/

     nonzero = 0;
     for (i=0; i < imdg->len; i++)
      if ((imdg->item[i] != NULL) && is_not_zero(imdg->item[i]))
	{ nonzero = 1;
	  break;
	}
     if (! nonzero)
     {   fprintf(Map,"\n%d %d 0\n",s1,g);
         status |= 0x100;
     }
     else	/*  imdg is nonzero.  We have to lift it.		*/
     {
       sprintf(filename,"../../%s/ToLift_%d.%d",Codomain,s1-s,t1-t);
       if ((Work = fopen(filename,"a")) == NULL)
       {  sprintf(message,"\nCannot open %s to append.\n",filename);
          terminate(message,BAD_WORKFILE);
       }
       fprintf(Work,"\n../%s/%s/Map %d %d \n",Domain,MapDir,s1,g);
       write_exp_Element(Work,&(imdg->item[0]),'c',imdg->len);
       fclose(Work);
       status |= 0x200;
     }

    return_whole_Element(dg);
    return_whole_Element(imdg);
} 	/* end if status not unmapped			*/
} 	/* end else g not mapped so compute map(dg)	*/
} 	/* end for g = 0 to dimCodomain			*/
fclose(Map);
}	/* end else s1 > s				*/

exit(status);
}

