/* -------------------------------------------------------------------	*/
/* Copyright (C) 2000 Robert R. Bruner.  All rights reserved.		*/
/* Contact the author at						*/
/*                    rrb@math.wayne.edu				*/
/*                    Dept of Mathematics				*/
/*                    Wayne State University				*/
/*                    Detroit, Michigan  48202				*/
/*                    USA						*/
/*									*/
/* Version 1.5								*/
/* -------------------------------------------------------------------- */

#include <steen.h>
#include <stdio.h>

/* Routines to define the subalgebra A(2) generated by Sq^1, Sq^2 and   */
/* Sq^4.   Some parts can clearly be generalized to A(n), but in the    */
/* interest of speed, this version also has a somewhat ad hoc aspect.   */

static int DEGREE[N_ALG] =
#if N_ALG == 1
     {1};
#endif
#if N_ALG == 2
     {1,3};
#endif
#if N_ALG == 3
     {1,3,7};
#endif
#if N_ALG == 4
     {1,3,7,15};
#endif
#if N_ALG == 5
     {1,3,7,15,31};
#endif
#if N_ALG == 6
     {1,3,7,15,31,63};
#endif
#if N_ALG == 7
     {1,3,7,15,31,63,127};
#endif

static int TopDegree = 24;	/* A(2) has top class in dim 23		*/
static int Dimension[24] =
       { 1,1,1,2,2,2,3,4,3,4,5,4,
         4,5,4,3,4,3,2,2,2,1,1,1  };

static Internal_Op Elements[24][5] =
       { { {0,0,0}, {0,0,0}, {0,0,0}, {0,0,0}, {0,0,0} }, /* deg  0 */
         { {1,0,0}, {0,0,0}, {0,0,0}, {0,0,0}, {0,0,0} }, /* deg  1 */
         { {2,0,0}, {0,0,0}, {0,0,0}, {0,0,0}, {0,0,0} }, /* deg  2 */
         { {3,0,0}, {0,1,0}, {0,0,0}, {0,0,0}, {0,0,0} }, /* deg  3 */
         { {4,0,0}, {1,1,0}, {0,0,0}, {0,0,0}, {0,0,0} }, /* deg  4 */
         { {5,0,0}, {2,1,0}, {0,0,0}, {0,0,0}, {0,0,0} }, /* deg  5 */
         { {6,0,0}, {3,1,0}, {0,2,0}, {0,0,0}, {0,0,0} }, /* deg  6 */
         { {7,0,0}, {4,1,0}, {1,2,0}, {0,0,1}, {0,0,0} }, /* deg  7 */
         { {5,1,0}, {2,2,0}, {1,0,1}, {0,0,0}, {0,0,0} }, /* deg  8 */
         { {6,1,0}, {3,2,0}, {0,3,0}, {2,0,1}, {0,0,0} }, /* deg  9 */
         { {7,1,0}, {4,2,0}, {1,3,0}, {3,0,1}, {0,1,1} }, /* deg 10 */
         { {5,2,0}, {2,3,0}, {4,0,1}, {1,1,1}, {0,0,0} }, /* deg 11 */
         { {6,2,0}, {3,3,0}, {5,0,1}, {2,1,1}, {0,0,0} }, /* deg 12 */
         { {7,2,0}, {4,3,0}, {6,0,1}, {3,1,1}, {0,2,1} }, /* deg 13 */
         { {5,3,0}, {7,0,1}, {4,1,1}, {1,2,1}, {0,0,0} }, /* deg 14 */
         { {6,3,0}, {5,1,1}, {2,2,1}, {0,0,0}, {0,0,0} }, /* deg 15 */
         { {7,3,0}, {6,1,1}, {3,2,1}, {0,3,1}, {0,0,0} }, /* deg 16 */
         { {7,1,1}, {4,2,1}, {1,3,1}, {0,0,0}, {0,0,0} }, /* deg 17 */
         { {5,2,1}, {2,3,1}, {0,0,0}, {0,0,0}, {0,0,0} }, /* deg 18 */
         { {6,2,1}, {3,3,1}, {0,0,0}, {0,0,0}, {0,0,0} }, /* deg 19 */
         { {7,2,1}, {4,3,1}, {0,0,0}, {0,0,0}, {0,0,0} }, /* deg 20 */
         { {5,3,1}, {0,0,0}, {0,0,0}, {0,0,0}, {0,0,0} }, /* deg 21 */
         { {6,3,1}, {0,0,0}, {0,0,0}, {0,0,0}, {0,0,0} }, /* deg 22 */
         { {7,3,1}, {0,0,0}, {0,0,0}, {0,0,0}, {0,0,0} }  /* deg 23 */
       };

static unsigned Sequence[2][4][8] =
   { { {0,0,0,0,0,0,0,0},
       {1,1,1,1,1,0,0,0},
       {2,2,1,1,1,0,0,0},
       {2,2,1,1,1,0,0,0} 
     },
     { {3,2,3,3,2,2,2,1},
       {4,3,3,3,2,1,1,0},
       {4,3,2,2,1,0,0,0},
       {3,2,1,1,1,0,0,0}
     }};                    /* Table lookup up for sequence numbers	*/

/* --------------------------------------------  degree     ---------- */
int degree(Op) Internal_Op Op;
{ int i,d=0;

for (i=0; i < N_ALG; i++)
   d += Op[i] * DEGREE[i];
return(d);
}

/* --------------------------------------------  Seqno    ---------------- */
unsigned Seqno(Op) Internal_Op Op; {
/*  Return the number of operations in this degree which come before Op    */
/*  in reverse lexicographic order.                                        */
unsigned s=Sequence[Op[2]][Op[1]][Op[0]];

return(s);
}

/* --------------------------------------------  Seqno    ---------------- */
unsigned dimA(n) int n; {
/*  Return the dimension of the algebra in degree n                        */

if ((n >= 0) && (n < TopDegree)) return(Dimension[n]);
else return(0);
}

/* --------------------------------------------  fprint_op   ---------- */
void fprint_op(fp,Op) FILE *fp; Internal_Op Op;
{ register int i,max;
  for (max=N_ALG; Op[--max]==0 && (max >0); );
  fprintf(fp,"(%d",Op[0]);
  for (i=1; i<=max; i++) fprintf(fp,",%d",Op[i]);
  fprintf(fp,")");
}

/* --------------------------------------------  print_op   ---------- */
void print_op(Op) Internal_Op Op;
{ fprint_op(stdout,Op); }

/* --------------------------------------------  first_op      ---------- */
int first_op(Op,deg) Internal_Op Op; int deg;
{ /* set the operation Op to the first Steenrod operation in degree    */
  /* deg, in (reverse) lexicographic order.  (Reverse because read     */
  /* right to left, not left to right: ie (3,0,0) < (0,1,0).           */
  /* Returns the sequence number of the first op, and sets Op to that  */
  /* operation, if there are any.  Otherwise returns -1                */

int i;

if ((deg >= 0) && (deg < TopDegree))
{ for (i=0; i<N_ALG; i++) 
         Op[i] = Elements[deg][0][i];
  return(0);
}
else return(-1);
}


/* ---------------------------------------  advance_op      ---------- */
int advance_op(Op) Internal_Op Op; {
/*  Advance Op to the lexically next operation in the same degree.          */
/*  (See first_op for comments on the order.) Return 1 if Op can be         */
/*  advanced, 0 if it is the last one in this degree.                       */

int seq,deg,i;

seq = Seqno(Op);
deg = degree(Op);

if (++seq < Dimension[deg])
{ for (i=0; i<N_ALG; i++)
     Op[i] = Elements[deg][seq][i];
  return(1);
}
else return(0);
}

/* --------------------------------------------  next_op      ---------- */
int next_op(Op,seq) Internal_Op Op; int seq; {
/*  Advance Op to the lexically next operation in the same degree.          */
/*  (See first_op for comments on the order.) Return it's sequence number,  */
/*  seq+1, if Op can be advanced.  Return -1 if it is the last one in     */
/*  this degree.                                                            */

int deg,i;

deg = degree(Op);

if (++seq < Dimension[deg])
{ for (i=0; i<N_ALG; i++)
     Op[i] = Elements[deg][seq][i];
  return(seq);
}
else return(-1);
}


/* --------------------------------------------  adv_op ------------------ */
int adv_op(Op,k) Internal_Op Op; int k; {
/*  Advance Op k steps in lexicographic order (k >= 0 here). Return 1 if   */
/* Op can be advanced, 0 if it would go past the end.                      */

int seq,deg,i;

seq = k + Seqno(Op);
deg = degree(Op);

if (seq < Dimension[deg])
{ for (i=0; i<N_ALG; i++)
     Op[i] = Elements[deg][seq][i];
  return(1);
}
else return(0);
}


/*  -----  routines used by mult and its global variables  -----------   */
/* -----------------------------------------------------------------     */


/* global variables */

int T[N_ALG+1],           /* hold terms of R * S, offset by one:           */
                          /*    T = (t1,t2,...,t[N]) (wasting T[0], but    */
                          /*    making the programming cleaner             */
    X[N_ALG+1][N_ALG+1],  /* matrix used to compute sum                    */
    N,maxrow,maxcol;      /* max nonzero diag, row and col                 */
/*  --------------------------------------------------------------------   */

int nextrow(row) int row;
{
                /* Try to advance row r, changing row 0 and col 0          */
                /* appropriately.  If successful, return 1.  If not,       */
                /* return 0 and reset row r to the start.                  */
int col, done=0;

for (col=1; (!done) && (col <= maxcol); col++)

    if (X[0][col] && (X[row][0] >> col))
      {                  /* can advance (row,col)                */
       X[row][col]++;
       X[row][0] -= 1 << col;
       X[0][col]--;
       done = 1;
      }
   else 
      {                /* can't advance it, so reset it.         */
       X[0][col] += X[row][col];
       X[row][0] += X[row][col] << col;
       X[row][col] = 0;
      }
return(done);
}

/*  --------------------------------------------------------------------   */
int next()
{
int row, done = 0;

for (row=1; (!done) && (row <= maxrow); row++)
  done = nextrow(row);
return(done);
}

/*  --------------------------------------------------------------------   */

int test()
{
int ok=1;
int diag, col, row, collim;

    for (diag=N; ok && (diag > 0); diag--)   
     { row = (maxrow < diag) ? maxrow : diag;
       T[diag] = X[row][col = diag - row];
       collim = (maxcol < diag) ? maxcol : diag;
       for (col++,row--; ok && (col <= collim); col++,row--)
         if (T[diag] & X[row][col]) ok = 0;
          else T[diag] += X[row][col];
     }
return(ok);
}

/* -------------------------------- mult      ---------------------------- */
int mult(R,S,x)   /* multiply two elements of the Steenrod algebra and add  */
                 /* the result to the operation in the TERM x.  We         */
                 /* assume they lie in the subalgebra generated by         */
                 /* Sq(2**i) for i = 0 to N_ALG - 1. Write N for N_ALG     */
		 /* in the rest of these coments.                          */

                 /* The vector R = (r0, r1, ..., r[N-1]) represents the    */
                 /* Milnor dual to the product of xi sub i to the r[i-1],  */
                 /* so must satisfy   0 <= r[N-i] < 2^i.  In particular,   */
                 /* r[0] < 2^N, so sq(2^{N-1}) is the highest              */
                 /* indecomposable Steenrod operation in the subalgebra.   */

/* The matrices X are ordered lexicographically by rows, with the first    */
/* row the least significant.  Rows are ordered lexicographically with     */
/* the column 1 entry the least significant.  The matrices are produced    */
/* in order.                                                               */

int *R, *S;
TERM *x;

{
int i, j, seq, notdone;

/* create initial matrix and set variables maxrow, maxcol, N and T         */

maxrow = maxcol = 1;
for (i=0; i < N_ALG; i++)
   { if ((X[0][i+1] = S[i])) maxcol = i+1;
     if ((X[i+1][0] = R[i])) maxrow = i+1;
     for (j=1; j <= N_ALG; j++) X[i+1][j] = 0;
   }
X[0][0] =0;
N = (N_ALG <= maxrow + maxcol) ? N_ALG : maxrow + maxcol;
for (i=N; i++ < N_ALG; ) T[i] = 0;



/* main loop  */

for (notdone=1; notdone; notdone=next())

  if (test())
  { 
    seq = Seqno(T+1);
    x->op[seq/8] ^= (0x80) >> (seq%8);
  }

return(0);
}




/* ----------------- first_Coproduct ---------------------------------- */

int first_Coproduct(Op,Op1,Op2) Internal_Op Op,Op1,Op2;
{
int i;

for (i=0; i< N_ALG; i++)
  { Op1[i] = 0;
    Op2[i] = Op[i];
  }
return(1);
}

/* ----------------- next_Coproduct ---------------------------------- */

int next_Coproduct(Op,Op1,Op2) Internal_Op Op,Op1,Op2;
{
int i;

for (i=0; i< N_ALG; i++)
  if (Op1[i] < Op[i]) { Op1[i]++; Op2[i]--; return(1); }
   else { Op1[i] = 0; Op2[i] = Op[i]; }
return(0);
}

