/* -------------------------------------------------------------------	*/
/* Copyright (C) 2000 Robert R. Bruner.  All rights reserved.		*/
/* Contact the author at						*/
/*                    rrb@math.wayne.edu				*/
/*                    Dept of Mathematics				*/
/*                    Wayne State University				*/
/*                    Detroit, Michigan  48202				*/
/*                    USA						*/
/*									*/
/* Version 1.6								*/
/* Version 1.9.2:  changed (char *) to (char **).  see note, line 378	*/
/* -------------------------------------------------------------------- */

/*#include <malloc.h> */
#include <stdlib.h>
#include <stdio.h>

#include <errors.h>
#include <structur.h>
#include <storage.h>
#include <steen.h>


#define OVERHEAD 8    /* malloc's overhead per block in bytes (or more)  */
                      /* 2 for PC, 4 for Sun 3, 8 for Sparc              */

#define ALLOC (65536 - OVERHEAD) /* amount to get from system each time  */
                                 /* 16K - OVERHEAD for PC,               */
                                 /* 64K - OVERHEAD for Suns              */

/* ----------- Basic structures for free list -------------------------- */

typedef struct freeblock
        {      unsigned len;                     /* length in UNITs      */
         struct freeblock *next;} freeblock;

#define UNIT sizeof(freeblock)

               /* all memory is allocated in multiples of UNIT bytes    */
               /* so that (by the Euclidean algorithm) no blocks        */
               /* smaller than UNIT will ever be formed.                */

static freeblock freelist =          /*  The Free List  (Ta Da)         */
                 {0,                 /* freelist.len is never used      */
                 NULL};              /* freelist.next is the free list  */


/* ------------------------- utility routine ---------------------- */

void printfree()
{ freeblock *f = &freelist;

while (f != NULL)
  { printf("\n%10u  %u",(unsigned)f,f->len);
    f = f->next;
  }
printf("\n ----- EOL ----- ");
}

/* -------------------  Basic allocation routines --------------------- */
/*      return_block       - return block to free list                  */
/*      get_block          - get new block from free list               */

/* Keep the free list in increasing order and amalgamate adjacent       */
/* blocks.  Allocate by first fit, carving the block from the end if    */
/* the fit is not exact.                                                */


void return_block(f) freeblock *f;
{ register freeblock *list = &freelist;

/* The routine never compares the address of f with that of list, so    */
/* the address of the variable free does not matter: it will always be  */
/* the root of the free list.  (The point is that it must be the root   */
/* no matter how its address compares to that of the blocks in the      */
/* free list.)                                                          */
/* Thus we may assume f is to be inserted after list.                   */

Top:

if (list+list->len > f) terminate("Returned block overlaps free block",
                                  BAD_MEMORY);

if (list->next == NULL)

       {  if (list + list->len == f)          /* splice f onto end of list */
                        list->len += f->len;
          else { list->next = f;
                 f->next = NULL;}
        }

else if (list->next < f) { list = list->next;
                           goto Top;
                          }

else if (list->next > f)
        {                         /* insert f between list and list->next */
          if (f + f->len == list->next)
            { if (list +list->len == f)

                {                      /* f,list and list->next contigous */
                   list->len += f->len + list->next->len;
                   list->next = list->next->next;
                }
              else {                      /* f and list->next contigous   */
                        f->next = list->next->next;
                        f->len += list->next->len;
                        list->next = f;
                   }
            }
          else { if (list + list->len == f)

                      {                  /* list and f contiguous       */
                        list->len += f->len;
                       }
                 else {                         /* no contiguity        */
                        f->next = list->next;
                        list->next = f;
                       }
               }
        }

else { fprintf(stderr,"\nError: trying to return a block already in freelist");
       fprintf(stderr,"\nAddress: %u",(unsigned)f);
       terminate("return_block error",BAD_MEMORY);
      }
return;
}                                               /* end of return_block  */

/* ----------------------- get_block ---------------------------------  */

freeblock * get_block(size) unsigned size;
                            /*  allocate a new free block whose length  */
                            /*  in UNIT's is given by size              */
{
freeblock *f,*x,*lastf;


Top:                                   /* find first block big enough   */

for (lastf = &freelist, f=freelist.next;
      f != NULL && f->len < size;
      lastf = f, f=f->next);

if (f == NULL) { /* none such: allocate some more or quit */
                 if (size > ALLOC / UNIT)
                       { fprintf(stderr,"\n block of size %u ",size);
                         terminate("Oversize block requested",
                                    OVERLARGE_BLOCK);
                       }
                 f = (freeblock *) malloc(ALLOC);
                 if (f == NULL) terminate("Out of memory",NO_MEMORY);
                 else {  f->len = ALLOC / UNIT;
                         f->next = NULL;
                       }
                 return_block(f);
                 goto Top;
               }

if (f->len == size)                       /* return whole block  */
     { lastf->next = f->next;
       return(f);
     }

else                                      /* return tail end of f   */
   { register unsigned i = f->len - size;
     f -> len = i;
     x = f+i;
     x -> len = size;
     return(x);
   }

}                                        /* end of get_block   */


/* ----- routines to allocate and free elements (and TERMs) ----------  */

/* Need ceilng function in the form ceiling(x,y) = ceiling of x/y       */

#define ceiling(x,y) 1 + (((x)-1)/y)

/* ------------------ get_TERM ---------------------------------------  */
TERM * get_TERM(len) unsigned len;
                            /*  allocate a new TERM whose operation     */
                            /*    field holds len ops (1 bit each)      */
                            /*    Its size in bytes is the ceiling of   */
                            /*         sizeof(TERM) + (len-8)/8         */
{
TERM  *x;
unsigned length;

if (len <= 8) length = sizeof(TERM);
 else length = sizeof(TERM) + 1 + (len-9)/8;

x = (TERM *) get_block(ceiling(length,UNIT));
x->len = len;
return(x);
}

/* ----------------- return_TERM -------------------------------------- */

void return_TERM(x) TERM *x;
{
freeblock *f = (freeblock *)x;
unsigned length;

if (x->len <= 8) length = sizeof(TERM);
 else length = sizeof(TERM) + 1 + (x->len-9)/8;

f->len = ceiling(length ,UNIT);
return_block(f);
return;
}

/* ----------------- return_TERMs -------------------------------------- */

void return_TERMs(x,dim) TERM *x[]; int dim;
{ int i;
for (i=0; i<dim; i++)
  if (x[i] != NULL) { return_TERM(x[i]);
                      x[i] = NULL;       }
}

/* ----------------- get_Element -------------------------------------- */

Element * get_Element(len) unsigned len;
                        /*  allocate a new Element with len TERMs       */
                        /*  Its size in bytes is                        */
                        /*  sizeof(Element) + (len-1) * sizeof(TERM *)  */
{
Element  *x;

if (len == 0) return(NULL);
x = (Element *) get_block(ceiling(sizeof(Element)+(len-1)*sizeof(TERM *),UNIT));
x->len = len;
return(x);
}



/* ----------------- return_Element -------------------------------------- */
/*                                    return only the top level of Element */
/*                                    to the freelist                      */

void return_Element(x) Element *x;
{
freeblock *f = (freeblock *)x;

f->len = ceiling(sizeof(Element)+(x->len-1)*sizeof(TERM *),UNIT);
return_block(f);
return;
}

/* ----------------- return_whole_Element --------------------------------- */
/*                                    return the top level and the TERMs    */
/*                                    of Element to the freelist            */

void return_whole_Element(x) Element *x;
{
freeblock *f = (freeblock *)x;

return_TERMs(x->item,x->len);
f->len = ceiling(sizeof(Element)+(x->len-1)*sizeof(TERM *),UNIT);
return_block(f);
return;
}

/* --------------------------------------------------------------------- */

/* ----- routines to allocate and free Tensors (and TTERMs) ----------  */

/* ------------------ get_TTERM ---------------------------------------  */
TTERM * get_TTERM()
                            /*  allocate a new TTERM                     */
{
return((TTERM *) get_block(ceiling(sizeof(TTERM),UNIT)));
}

/* ----------------- return_TTERM -------------------------------------- */

void return_TTERM(x) TTERM *x;
{
freeblock *f = (freeblock *)x;

f->len = ceiling(sizeof(TTERM),UNIT);
return_block(f);
return;
}

/* ----------------- get_Tensor -------------------------------------- */

Tensor * get_Tensor(len) unsigned len;
                        /*  allocate a new Tensor with len TTERMs       */
                        /*  Its size in bytes is                        */
                        /*  sizeof(Tensor) + (len-1) * sizeof(TTERM *)  */
{
Tensor  *x;

if (len == 0) return(NULL);
x = (Tensor *) get_block(ceiling(sizeof(Tensor)+(len-1)*sizeof(TTERM *),UNIT));
x->len = len;
return(x);
}



/* ----------------- return_whole_Tensor -------------------------------- */

void return_whole_Tensor(x) Tensor *x;
{
int i;
freeblock *f = (freeblock *)x;

for (i=0; i < x->len; i++)
  { return_whole_Element(x->item[i]->RHS);
    return_TTERM(x->item[i]);
  }

f->len = ceiling(sizeof(Tensor)+(x->len-1)*sizeof(TTERM *),UNIT);
return_block(f);
return;
}

/* ----------------- return_Tensor -------------------------------------- */

void return_Tensor(x) Tensor *x;
{
freeblock *f = (freeblock *)x;

if (x == NULL) return;
f->len = ceiling(sizeof(Tensor)+(x->len-1)*sizeof(TTERM *),UNIT);
return_block(f);
return;
}

/* --------------------------------------------------------------------- */
/* expanded tensors need blocks of pointers                              */
/*   These are typed (char **), so must be cast to the appropriate type. */
/* --------------------------------------------------------------------- */

char ** get_pointers(len) int len;
{
char **x;

if (len < 0) terminate("Negative length in get_pointers",BAD_NODE);
x = (char **) get_block(ceiling(len*sizeof(char **),UNIT));
return(x);
}

/* --------------------------------------------------------------------- */

void return_pointers(x,len) char **x; unsigned len;
{
freeblock *f = (freeblock *)x;

f->len = ceiling(len*sizeof(char **),UNIT);
return_block(f);
return;
}

/* --------------------------------------------------------------------- */

void return_exp_Tensor(e,pe_shape) Element *****e;
			      Tensor_shape *pe_shape;
{ int s,opdeg,g,seq;

if (e != NULL)
{ for (s=0; s<= pe_shape->s; s++)
  if (e[s] != NULL)
   { for (g=0; g < pe_shape->numgen[s]; g++)
      if (e[s][g] != NULL)
      { for (opdeg=0; opdeg <= pe_shape->t - pe_shape->deg[s][g]; opdeg++)
	  if (e[s][g][opdeg] != NULL)
	  { for (seq=0; seq < dimA(opdeg); seq++)
	      if (e[s][g][opdeg][seq] != NULL) 
                return_whole_Element(e[s][g][opdeg][seq]);
            return_pointers((char **)e[s][g][opdeg],dimA(opdeg));
	  }
        return_pointers((char **)e[s][g], 1+pe_shape->t - pe_shape->deg[s][g]);
	/* Comment in version 1.9.2:          					   */
	/* is this really (char *) or should it be (char **) like all the others?  */
	/* I have changed it to avoid error messages, but note that this is not used
	   anywhere in the code.                                                   */
      }
     return_pointers((char **)e[s], pe_shape->numgen[s]);
   }
return_pointers((char **)e, 1+pe_shape->s);
}
return;
}


